# encoding: UTF-8
require 'rubygems'
require 'pp'
require 'net/http'
require 'net/https'
require 'uri'
require 'json'
require 'openssl'
require 'digest/sha1'
require 'securerandom'
require 'base64'
require 'open-uri'

OpenSSL::SSL::VERIFY_PEER = OpenSSL::SSL::VERIFY_NONE

def getApi(protocol, server, port, path, token)
        url="#{protocol}://#{server}#{port}#{path}"
        puts "url: #{url}"
        uri = URI.parse(url)
        prot = Net::HTTP.new(uri.host, uri.port)
        prot.read_timeout = 6000
		if (protocol == "https") then
			prot.use_ssl = true
			prot.verify_mode = OpenSSL::SSL::VERIFY_NONE
		end
        header = {'OPSWAT-Token' => token}
        request = nil
        response = nil
        request = Net::HTTP::Get.new(uri.request_uri, header)
        response = prot.request(request)
        if(response.code.to_i != 200) then
                puts "ERROR: Data generation aborted! The query '#{url}' failed! Code:#{response.code.to_s}"
                puts "response:#{response.body.to_json}"
                exit! false
        end
        return JSON.parse(response.body)
end

def postApi(protocol, server, port, path, token, body, ctype = 'text/plain')
        url="#{protocol}://#{server}#{port}#{path}"
        puts "url: #{url}"
        uri = URI.parse(url)
        prot = Net::HTTP.new(uri.host, uri.port)
        prot.read_timeout = 6000
		if (protocol == "https") then
			prot.use_ssl = true
			prot.verify_mode = OpenSSL::SSL::VERIFY_NONE
		end
        header = {'OPSWAT-Token' => token, 'Content-Type' => ctype}
        request = Net::HTTP::Post.new(uri.request_uri, header)
        if (body.is_a?(String)) then
          request.body = body
        else
          request.body = body.to_json
        end
        response = prot.request(request)
        if(response.code.to_i != 200) then
                puts "ERROR: file registry aborted! The query '#{url}' failed! Code:#{response.code.to_s}"
                puts "req:#{body.to_json}"
                puts "response:#{response.body.to_json}"
                exit! false
        end
        return JSON.parse(response.body)
end

def makeGatewayPostNewKey(gateway, port, protocol, token, keyFile)
  path = "/api/file-retrieval/update-key"
  filecontent = File.read(keyFile)
  key = filecontent.gsub('-----END PUBLIC KEY-----', '').gsub('-----BEGIN PUBLIC KEY-----', '')
  return postApi(protocol, gateway, port, path, token, key)
end

def makeGatewayRequestForMeta(gateway, port, protocol, token, file)
  path = "/api/file-retrieval/file-link/#{file}"
  return getApi(protocol, gateway, port, path, token)
end

def getKeyFromFile(keyFile)
  filecontent = File.read(keyFile)
  return OpenSSL::PKey::RSA.new(filecontent)
end

def decryptMeta(private_key, cryptMeta)
  chunk = cryptMeta.split('_')
  decData = ""
  chunk.each{ |c| decData << private_key.private_decrypt(Base64.decode64(c)) }
  return decData
end

def getMetaFromGateway(gateway, port, protocol, token, privateKeyFile, file)
  if (privateKeyFile) then
    cryptMeta = makeGatewayRequestForMeta(gateway, port, protocol, token, file)
    puts "result: #{cryptMeta}"
    privKey = getKeyFromFile(privateKeyFile)
    return JSON.parse(decryptMeta(privKey, cryptMeta["result"]))
  else
    meta = makeGatewayRequestForMeta(gateway, port, protocol, token, file)
    puts "result: #{meta}"
    return meta["result"]
  end
end

def downloadLinkedFile(url, fpath)
  File.open(fpath, "w") do |f|
    IO.copy_stream(open(url), f)
  end
end

def decryptFile(file_read, file_write, key)
  f1 = File.new(file_read, "rb")
  data = f1.read()
  f1.close
  cipher = OpenSSL::Cipher::AES256.new(:CBC)
  dkey = Base64.decode64(key)
  cipher.key = dkey
  decrypted = cipher.update(data) + cipher.final
  f2 = File.new(file_write, "wb")
  f2.write(decrypted)
  f2.close
end


def execute(gateway, port, protocol, token, privKeyFile, file)
      begin
            meta = getMetaFromGateway(gateway, port, protocol, token, privKeyFile, file)
            puts "meta: #{meta}"
            if (meta) then
              if meta["aes_key"] then
                encFile = meta["link"].split('/').last
                downloadLinkedFile(meta["link"], encFile)
                decryptFile(encFile, meta["file"], meta["aes_key"])
                File.delete encFile
              else
                downloadLinkedFile(meta["link"], meta["file"])
              end
            end
      rescue Exception => e
            puts "File client failed due to the following error:\n#{e}"
            exit! false
      end
      puts "Done"
end

def showHelp
  puts "--help                     show this help menu"
  puts "--gateway [gateway server] specify non-local host server address"
  puts "--port [port]              specify non-default port"
  puts "--token [token]            specify OPSWAT-Token"
  puts "--file [file]              name of the file to download"
  puts "--protocol [protocol]      specify http instead of https"
  puts "--upload-public-key        upload new public key to gateway"
  puts "--keyfile [file]           specify public/private key file for encryption/decryption"
end

def parseArgs
    gateway = "oesis.metadefender.com"
    port = ""
    noport = true
    token = nil
    protocol = "https"
    upkey = false
    keyFile = nil
    dFile = nil
    while arg = ARGV.shift do
      case arg
          when /--gateway/i then gateway = ARGV.shift
          when /--port/i then port = ARGV.shift; noport = false;
          when /--token/i then token = ARGV.shift
          when /--protocol/i then protocol = ARGV.shift
          when /--upload-public-key/i then upkey = true
          when /--keyfile/i then keyFile = ARGV.shift
          when /--file/i then dFile = ARGV.shift
          when /--help/i then showHelp; exit! true
      end
    end
    port = ":#{port}"
    if noport then
      port = ""
    end
    if upkey then
      makeGatewayPostNewKey(gateway, port, protocol, token, keyFile)
    else
      execute(gateway, port, protocol, token, keyFile, dFile)
    end
end

parseArgs
